library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Init_all is
    Port ( rst : in std_logic;
           ck : in std_logic;
		 --Signals for the UART
		 UART_addr_bus: out std_logic_vector(1 downto 0);
		 UART_data_bus: out std_logic_vector(7 downto 0);
   	 UART_data_bus_rx: in std_logic_vector(7 downto 0);
		 UART_wr_rd: out std_logic;
		 UART_tr_cycle: out std_logic;
		 UART_tx_int: in std_logic;
		 UART_rx_int: in std_logic;
		 UART_ack: in std_logic
	     );
end Init_all;

architecture Behavioral of Init_all is

	  type state_type is (sa, sb, sc, sd, s0,s1,st1_wait,s2,s3,s3delay,s3nitx);   
	  signal state: state_type ; 
	 	 
	  signal rx_data : std_logic_vector(7 downto 0);
	  
	  
	  --record/playback enabled
	  type wordarrays_1 is array(0 to 26) of std_logic_vector(7 downto 0);
	  constant record_en_words : wordarrays_1 := ( 
		X"52", X"65", X"63", X"6F", X"72", X"64", X"2F", X"70", X"6C", X"61",
		X"79", X"62", X"61", X"63", X"6B", X"20", X"45", X"6E", X"61", X"62",
		X"6C", X"65", X"64" ,X"0A" ,X"0D", X"0A", X"0D");
		
		--record/playback disabled
	  type wordarrays_2 is array(0 to 27) of std_logic_vector(7 downto 0);
	  constant record_dis_words : wordarrays_2 := ( 
		X"52", X"65", X"63", X"6F", X"72", X"64", X"2F", X"70", X"6C", X"61",
		X"79", X"62", X"61", X"63", X"6B", X"20", X"44", X"69", X"73", X"61",
		X"62", X"6C", X"65", X"64", X"0A" ,X"0D", X"0A", X"0D");
				
	  --erase enabled
	  type wordarrays_3 is array(0 to 16) of std_logic_vector(7 downto 0);
     constant erase_en_words : wordarrays_3 := ( 
		X"45", X"72", X"61", X"73", X"65", X"20", X"45", X"6E", X"61", X"62",
		X"6C", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
	  --erase disabled
	  type wordarrays_4 is array(0 to 17) of std_logic_vector(7 downto 0);
     constant erase_dis_words : wordarrays_4 := ( 
		X"45", X"72", X"61", X"73", X"65", X"20", X"44", X"69", X"73", X"61",
		X"62", X"6C", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
		--recording started
	  type wordarrays_5 is array(0 to 20) of std_logic_vector(7 downto 0);
     constant record_start_words : wordarrays_5 := ( 
		X"52", X"65", X"63", X"6F", X"72", X"64", X"69", X"6E", X"67", X"20",
		X"53", X"74", X"61", X"72", X"74", X"65", X"64", X"0A", X"0D", X"0A",
		X"0D");
		
		--recording stopped
	  type wordarrays_6 is array(0 to 20) of std_logic_vector(7 downto 0);
     constant record_stop_words : wordarrays_6 := ( 
		X"52", X"65", X"63", X"6F", X"72", X"64", X"69", X"6E", X"67", X"20",
		X"53", X"74", X"6F", X"70", X"70", X"65", X"64", X"0A", X"0D", X"0A",
		X"0D");
		
		--memory erased
	  type wordarrays_7 is array(0 to 16) of std_logic_vector(7 downto 0);
     constant memory_erased_words : wordarrays_7 := ( 
		X"4D", X"65", X"6D", X"6F", X"72", X"79", X"20", X"45", X"72", X"61",
		X"73", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
		--playback started
	  type wordarrays_8 is array(0 to 19) of std_logic_vector(7 downto 0);
     constant play_started_words : wordarrays_8 := ( 
		X"50", X"6C", X"61", X"79", X"62", X"61", X"63", X"6B", X"20", X"53",
		X"74", X"61", X"72", X"74", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
		--playback stopped
	  type wordarrays_9 is array(0 to 19) of std_logic_vector(7 downto 0);
     constant play_stopped_words : wordarrays_9 := ( 
		X"50", X"6C", X"61", X"79", X"62", X"61", X"63", X"6B", X"20", X"53",
		X"74", X"6F", X"70", X"70", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
		--microphone on
	  type wordarrays_10 is array(0 to 16) of std_logic_vector(7 downto 0);
     constant mic_on_words : wordarrays_10 := ( 
		X"4D", X"69", X"63", X"72", X"6F", X"70", X"68", X"6F", X"6E", X"65",
		X"20", X"4F", X"6E", X"0A", X"0D", X"0A", X"0D");
		
		--microphone off
	  type wordarrays_11 is array(0 to 17) of std_logic_vector(7 downto 0);
     constant mic_off_words : wordarrays_11 := ( 
		X"4D", X"69", X"63", X"72", X"6F", X"70", X"68", X"6F", X"6E", X"65",
		X"20", X"4F", X"66", X"66", X"0A", X"0D", X"0A", X"0D");
		
		--can't record - mic off
	  type wordarrays_12 is array(0 to 25) of std_logic_vector(7 downto 0);
     constant cant_record_mic_words : wordarrays_12 := ( 
		X"43", X"61", X"6E", X"27", X"74", X"20", X"52", X"65", X"63", X"6F",
		X"72", X"64", X"20", X"2D", X"20", X"4D", X"69", X"63", X"20", X"4F",
		X"66", X"66", X"0A", X"0D", X"0A", X"0D");
		
		--can't record - memory full
	  type wordarrays_13 is array(0 to 29) of std_logic_vector(7 downto 0);
     constant cant_record_memory_words : wordarrays_13 := ( 
		X"43", X"61", X"6E", X"27", X"74", X"20", X"52", X"65", X"63", X"6F",
		X"72", X"64", X"20", X"2D", X"20", X"4D", X"65", X"6D", X"6F", X"72",
		X"79", X"20", X"46", X"75", X"6C", X"6C", X"0A", X"0D", X"0A", X"0D");
		
		--can't playback - memory erased
	  type wordarrays_14 is array(0 to 33) of std_logic_vector(7 downto 0);
     constant cant_play_mem_erased_words : wordarrays_14 := ( 
		X"43", X"61", X"6E", X"27", X"74", X"20", X"50", X"6C", X"61", X"79",
		X"62", X"61", X"63", X"6B", X"20", X"2D", X"20", X"4D", X"65", X"6D",
		X"6F", X"72", X"79", X"20", X"45", X"72", X"61", X"73", X"65", X"64",
		X"0A", X"0D", X"0A", X"0D");
		
		--error - record/playback not enabled
	  type wordarrays_15 is array(0 to 38) of std_logic_vector(7 downto 0);
     constant err_rec_pl_not_enabled_words : wordarrays_15 := ( 
		X"45", X"72", X"72", X"6F", X"72", X"20", X"2D", X"20", X"52", X"65",
		X"63", X"6F", X"72", X"64", X"2F", X"70", X"6C", X"61", X"79", X"62",
		X"61", X"63", X"6B", X"20", X"4E", X"6F", X"74", X"20", X"45", X"6E",
		X"61", X"62", X"6C", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
		--error - erase not enabled
	  type wordarrays_16 is array(0 to 28) of std_logic_vector(7 downto 0);
     constant err_erase_not_enabled_words : wordarrays_16 := ( 
		X"45", X"72", X"72", X"6F", X"72", X"20", X"2D", X"20", X"45", X"72",
		X"61", X"73", X"65", X"20", X"4E", X"6F", X"74", X"20", X"45", X"6E",
		X"61", X"62", X"6C", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
		--error - erase still enabled
	  type wordarrays_17 is array(0 to 30) of std_logic_vector(7 downto 0);
     constant err_erase_still_enabled_words : wordarrays_17 := ( 
		X"45", X"72", X"72", X"6F", X"72", X"20", X"2D", X"20", X"45", X"72",
		X"61", X"73", X"65", X"20", X"53", X"74", X"69", X"6C", X"6C", X"20",
		X"45", X"6E", X"61", X"62", X"6C", X"65", X"64", X"0A", X"0D", X"0A",
		X"0D");
		
		--error - record/playback still enabled
	  type wordarrays_18 is array(0 to 40) of std_logic_vector(7 downto 0);
     constant rec_pl_still_enabled_words : wordarrays_18 := ( 
		X"45", X"72", X"72", X"6F", X"72", X"20", X"2D", X"20", X"52", X"65",
		X"63", X"6F", X"72", X"64", X"2F", X"70", X"6C", X"61", X"79", X"62",
		X"61", X"63", X"6B", X"20", X"53", X"74", X"69", X"6C", X"6C", X"20",
		X"45", X"6E", X"61", X"62", X"6C", X"65", X"64", X"0A", X"0D", X"0A",
		X"0D");


	  signal outputlength, outputcounter, choiceout : integer := 0;
	  
	  --signals to represent the old inputs from the fpga switches and buttons
	  signal switch_in1_main, switch_in2_main, switch_in3_main, switch_in4_main,
	  record_button_main, play_button_main, erase_button_main : std_logic := '0';
	  
	  COMPONENT clock_dcm
		PORT(
		CLKIN_IN : IN std_logic;
		RST_IN : IN std_logic;          
		CLKFX_OUT : OUT std_logic;
		CLKIN_IBUFG_OUT : OUT std_logic;
		CLK0_OUT : OUT std_logic;
		LOCKED_OUT : OUT std_logic
		);
		END COMPONENT;
		
		signal Clk_Div, dcm_lock : std_logic;

	  
begin

Inst_clock_dcm: clock_dcm 
    PORT MAP(
		CLKIN_IN => ck,
		RST_IN => rst,
		CLKFX_OUT => Clk_Div,
		CLKIN_IBUFG_OUT => open,
		CLK0_OUT => open,
		LOCKED_OUT => dcm_lock
	);

  process (ck,rst)
  
  begin 

    if (rst ='1') then
      -- Initialization of state
      state <= sa;
	   -- Initialization of UART
	   UART_addr_bus <= "00";
	   UART_data_bus <= "00000000";
	   UART_wr_rd <= '1';
	   UART_tr_cycle <='0';
	 
    elsif (ck='1' and ck'Event) then
      case state is

	   when sa =>
			if UART_rx_int = '1' then
				state <= sb;	
			else
				state <= sa;	-- Waiting until the UART is ready to RX
			end if;
			
	   when sb => state <= sc;   -- Delay to allow pin int up.

	   when sc => 
			if UART_rx_int = '1' then	-- Rx 
				state <= sd;    
				UART_addr_bus <= "00";
				UART_wr_rd <= '0';
				rx_data  <=  UART_data_bus_rx; 
				UART_tr_cycle <= '1';
         else 
				state <= sc;
         end if;
	              
        when sd => 
		  
  --ok well the first thing we need to do is assign keys to each button
  --and switch that we used from the previous program to control the 
  --recording functions
  
  -- r = record button
  -- e = erase button
  -- p = playback button
  
  -- 1 = mic mute or unmute
  -- 2 = erase enable or disable
  -- 3 = record/playback enable or disable
		  
if UART_ack = '1' then  --Wait for ack

   if rx_data = X"72" then -- Comparing for rx an "r"
	   outputlength <= wordarrays_1'length - 1;
	   outputcounter <= 0;
	   choiceout <= 1;
	   state <= s0;
		
   elsif rx_data = X"65" then -- Comparing for rx an "e"
	   outputlength <= wordarrays_2'length - 1;
	   outputcounter <= 0;
	   choiceout <= 2;
	   state <= s0;
		
   elsif rx_data = X"70" then -- Comparing for rx an "p"
	   outputlength <= wordarrays_3'length - 1;
	   outputcounter <= 0;
      choiceout <= 3;
	   state <= s0;
		
	elsif rx_data = X"31" then -- Comparing for rx an "1"
	   if switch_in2_main = '0' then
			switch_in2_main <= '1';
			outputlength <= wordarrays_10'length - 1;
			choiceout <= 10;
			
		else
			switch_in2_main <= '0';
			outputlength <= wordarrays_11'length - 1;
			choiceout <= 11;
			
		end if;
		
		outputcounter <= 0;
	   state <= s0;
	
	elsif rx_data = X"32" then -- Comparing for rx an "2"
	   if switch_in3_main = '0' then
			switch_in3_main <= '1';
			outputlength <= wordarrays_3'length - 1;
			choiceout <= 3;
			
		else
			switch_in3_main <= '0';
			outputlength <= wordarrays_4'length - 1;
			choiceout <= 4;
			
		end if;
		
		outputcounter <= 0;
	   state <= s0;
		
	elsif rx_data = X"33" then -- Comparing for rx an "3"
	   if switch_in4_main = '0' then
			switch_in4_main <= '1';
			outputlength <= wordarrays_1'length - 1;
			choiceout <= 1;
			
		else
			switch_in4_main <= '0';
			outputlength <= wordarrays_2'length - 1;
			choiceout <= 2;
			
		end if;
		
		outputcounter <= 0;
	   state <= s0;
		
	else
	   state <= sa;
	end if;
	
 UART_tr_cycle <= '0';
 
else 
 state <= sd;
end if;	  
			   
	
		when s0 => state <= s1; 	-- Delay to allow pin int UART down
 
	   when s1 => -- Starting the machine. Checking for End of Conversion.

				 state <= st1_wait;
				 -- Keeping UART IDLE
			    UART_addr_bus <= "00";
	 		    UART_data_bus <= "00000000";
	 		    UART_wr_rd <= '1';
	 		    UART_tr_cycle <='0';
				    
	   when st1_wait =>
			if UART_tx_int = '1' then 
				state <= s2;
			else
				state <= st1_wait;
			end if;
		
		when s2 => state <= s3;  --Tx first nibble
	   		 UART_addr_bus <= "00"; 
			
			case choiceout is
			
			when 1 =>
				UART_data_bus <= record_en_words(outputcounter);
			when 2 =>
				UART_data_bus <= record_dis_words(outputcounter);
			when 3 =>
				UART_data_bus <= erase_en_words(outputcounter);
			when 4 =>
				UART_data_bus <= erase_dis_words(outputcounter);
			when 5 =>
				UART_data_bus <= record_start_words(outputcounter);
			when 6 =>
				UART_data_bus <= record_stop_words(outputcounter);
			when 7 =>
				UART_data_bus <= memory_erased_words(outputcounter);
			when 8 =>
				UART_data_bus <= play_started_words(outputcounter);
			when 9 =>
				UART_data_bus <= play_stopped_words(outputcounter);
			when 10 =>
				UART_data_bus <= mic_on_words(outputcounter);
			when 11 =>
				UART_data_bus <= mic_off_words(outputcounter);
			when 12 =>
				UART_data_bus <= cant_record_mic_words(outputcounter);
			when 13 =>
				UART_data_bus <= cant_record_memory_words(outputcounter);
			when 14 =>
				UART_data_bus <= cant_play_mem_erased_words(outputcounter);
			when 15 =>
				UART_data_bus <= err_rec_pl_not_enabled_words(outputcounter);
			when 16 =>
				UART_data_bus <= err_erase_not_enabled_words(outputcounter);
			when 17 =>
				UART_data_bus <= err_erase_still_enabled_words(outputcounter);
			when 18 =>
				UART_data_bus <= rec_pl_still_enabled_words(outputcounter);
			when others =>
				--nothing here
			end case;				
				
	 		    UART_wr_rd <= '1';
	 		    UART_tr_cycle <='1';  -- For WishBone protocol

	   when s3 => 
			if UART_ack = '1' then  -- Waiting for UART ack
				state <= s3delay;	 
			   UART_addr_bus <= "00";
	 		      
			case choiceout is
			
			when 1 =>
				UART_data_bus <= record_en_words(outputcounter);
			when 2 =>
				UART_data_bus <= record_dis_words(outputcounter);
			when 3 =>
				UART_data_bus <= erase_en_words(outputcounter);
			when 4 =>
				UART_data_bus <= erase_dis_words(outputcounter);
			when 5 =>
				UART_data_bus <= record_start_words(outputcounter);
			when 6 =>
				UART_data_bus <= record_stop_words(outputcounter);
			when 7 =>
				UART_data_bus <= memory_erased_words(outputcounter);
			when 8 =>
				UART_data_bus <= play_started_words(outputcounter);
			when 9 =>
				UART_data_bus <= play_stopped_words(outputcounter);
			when 10 =>
				UART_data_bus <= mic_on_words(outputcounter);
			when 11 =>
				UART_data_bus <= mic_off_words(outputcounter);
			when 12 =>
				UART_data_bus <= cant_record_mic_words(outputcounter);
			when 13 =>
				UART_data_bus <= cant_record_memory_words(outputcounter);
			when 14 =>
				UART_data_bus <= cant_play_mem_erased_words(outputcounter);
			when 15 =>
				UART_data_bus <= err_rec_pl_not_enabled_words(outputcounter);
			when 16 =>
				UART_data_bus <= err_erase_not_enabled_words(outputcounter);
			when 17 =>
				UART_data_bus <= err_erase_still_enabled_words(outputcounter);
			when 18 =>
				UART_data_bus <= rec_pl_still_enabled_words(outputcounter);
			when others =>
				--nothing here
			end case;				
			    
	 		   UART_wr_rd <= '1';
	 		   UART_tr_cycle <='0'; -- For WishBone protocol
	   	else
			   state <= s3; -- Waiting until for UART confirmation of WishBone
         end if;

        when s3delay =>  --Delay
	              state <= s3NiTx; --Delay to allow pin int UART down.
					  outputcounter <= outputcounter + 1;

	   when s3NiTx => 
				if UART_tx_int = '1' and outputcounter <= outputlength then
				  state <= s0;
				elsif UART_tx_int = '1' and outputcounter > outputlength then
				  outputcounter <= 0;
				  state <= sa;
				else
			     state <= s3NiTx;	-- Waiting until the UART is ready to TX
            end if;

      end case;
    end if; 

  end process;
end Behavioral;
