library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Init_all is
    Port ( rst : in std_logic;
           ck : in std_logic;  --this is the 100 Mhz clk
			  div_clk : in std_logic;  --this is the 24 Mhz clk
			  dcm_lock : in std_logic;
		 --Signals for the UART
		 UART_addr_bus: out std_logic_vector(1 downto 0);
		 UART_data_bus: out std_logic_vector(7 downto 0);
   	 UART_data_bus_rx: in std_logic_vector(7 downto 0);
		 UART_wr_rd: out std_logic;
		 UART_tr_cycle: out std_logic;
		 UART_tx_int: in std_logic;
		 UART_rx_int: in std_logic;
		 UART_ack: in std_logic;
		 
		 -- CODEC signals
		 AC97Reset_n : out std_logic;
		 AC97Clk   : in  std_logic;   -- master clock for design
		 Sync      : out std_logic;
		 SData_Out : out std_logic;
		 SData_In  : in  std_logic;
		 
		 LED_OUTPUT1_main 	: out std_logic; --output to the led1
		 LED_OUTPUT2_main 	: out std_logic; --output to the led2
		 LED_OUTPUT3_main 	: out std_logic; --output to the led3
		 LED_OUTPUT4_main 	: out std_logic --output to the led4
	     );
end Init_all;

architecture Behavioral of Init_all is

	  type state_type is (sa, sb, sc, sd, s0,s1,st1_wait,s2,s3,s3delay,s3nitx);   
	  signal state: state_type ; 
	 	 
	  signal rx_data : std_logic_vector(7 downto 0);
	  
	  
	  --record/playback enabled
	  type wordarrays_1 is array(0 to 26) of std_logic_vector(7 downto 0);
	  constant record_en_words : wordarrays_1 := ( 
		X"52", X"65", X"63", X"6F", X"72", X"64", X"2F", X"70", X"6C", X"61",
		X"79", X"62", X"61", X"63", X"6B", X"20", X"45", X"6E", X"61", X"62",
		X"6C", X"65", X"64" ,X"0A" ,X"0D", X"0A", X"0D");
		
		--record/playback disabled
	  type wordarrays_2 is array(0 to 27) of std_logic_vector(7 downto 0);
	  constant record_dis_words : wordarrays_2 := ( 
		X"52", X"65", X"63", X"6F", X"72", X"64", X"2F", X"70", X"6C", X"61",
		X"79", X"62", X"61", X"63", X"6B", X"20", X"44", X"69", X"73", X"61",
		X"62", X"6C", X"65", X"64", X"0A" ,X"0D", X"0A", X"0D");
				
	  --erase enabled
	  type wordarrays_3 is array(0 to 16) of std_logic_vector(7 downto 0);
     constant erase_en_words : wordarrays_3 := ( 
		X"45", X"72", X"61", X"73", X"65", X"20", X"45", X"6E", X"61", X"62",
		X"6C", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
	  --erase disabled
	  type wordarrays_4 is array(0 to 17) of std_logic_vector(7 downto 0);
     constant erase_dis_words : wordarrays_4 := ( 
		X"45", X"72", X"61", X"73", X"65", X"20", X"44", X"69", X"73", X"61",
		X"62", X"6C", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
		--recording started
	  type wordarrays_5 is array(0 to 20) of std_logic_vector(7 downto 0);
     constant record_start_words : wordarrays_5 := ( 
		X"52", X"65", X"63", X"6F", X"72", X"64", X"69", X"6E", X"67", X"20",
		X"53", X"74", X"61", X"72", X"74", X"65", X"64", X"0A", X"0D", X"0A",
		X"0D");
		
		--recording stopped
	  type wordarrays_6 is array(0 to 20) of std_logic_vector(7 downto 0);
     constant record_stop_words : wordarrays_6 := ( 
		X"52", X"65", X"63", X"6F", X"72", X"64", X"69", X"6E", X"67", X"20",
		X"53", X"74", X"6F", X"70", X"70", X"65", X"64", X"0A", X"0D", X"0A",
		X"0D");
		
		--memory erased
	  type wordarrays_7 is array(0 to 16) of std_logic_vector(7 downto 0);
     constant memory_erased_words : wordarrays_7 := ( 
		X"4D", X"65", X"6D", X"6F", X"72", X"79", X"20", X"45", X"72", X"61",
		X"73", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
		--playback started
	  type wordarrays_8 is array(0 to 19) of std_logic_vector(7 downto 0);
     constant play_started_words : wordarrays_8 := ( 
		X"50", X"6C", X"61", X"79", X"62", X"61", X"63", X"6B", X"20", X"53",
		X"74", X"61", X"72", X"74", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
		--playback stopped
	  type wordarrays_9 is array(0 to 19) of std_logic_vector(7 downto 0);
     constant play_stopped_words : wordarrays_9 := ( 
		X"50", X"6C", X"61", X"79", X"62", X"61", X"63", X"6B", X"20", X"53",
		X"74", X"6F", X"70", X"70", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
		--microphone on
	  type wordarrays_10 is array(0 to 16) of std_logic_vector(7 downto 0);
     constant mic_on_words : wordarrays_10 := ( 
		X"4D", X"69", X"63", X"72", X"6F", X"70", X"68", X"6F", X"6E", X"65",
		X"20", X"4F", X"6E", X"0A", X"0D", X"0A", X"0D");
		
		--microphone off
	  type wordarrays_11 is array(0 to 17) of std_logic_vector(7 downto 0);
     constant mic_off_words : wordarrays_11 := ( 
		X"4D", X"69", X"63", X"72", X"6F", X"70", X"68", X"6F", X"6E", X"65",
		X"20", X"4F", X"66", X"66", X"0A", X"0D", X"0A", X"0D");
		
		--can't record - mic off
	  type wordarrays_12 is array(0 to 25) of std_logic_vector(7 downto 0);
     constant cant_record_mic_words : wordarrays_12 := ( 
		X"43", X"61", X"6E", X"27", X"74", X"20", X"52", X"65", X"63", X"6F",
		X"72", X"64", X"20", X"2D", X"20", X"4D", X"69", X"63", X"20", X"4F",
		X"66", X"66", X"0A", X"0D", X"0A", X"0D");
		
		--can't record - memory full
	  type wordarrays_13 is array(0 to 29) of std_logic_vector(7 downto 0);
     constant cant_record_memory_words : wordarrays_13 := ( 
		X"43", X"61", X"6E", X"27", X"74", X"20", X"52", X"65", X"63", X"6F",
		X"72", X"64", X"20", X"2D", X"20", X"4D", X"65", X"6D", X"6F", X"72",
		X"79", X"20", X"46", X"75", X"6C", X"6C", X"0A", X"0D", X"0A", X"0D");
		
		--can't playback - memory erased
	  type wordarrays_14 is array(0 to 33) of std_logic_vector(7 downto 0);
     constant cant_play_mem_erased_words : wordarrays_14 := ( 
		X"43", X"61", X"6E", X"27", X"74", X"20", X"50", X"6C", X"61", X"79",
		X"62", X"61", X"63", X"6B", X"20", X"2D", X"20", X"4D", X"65", X"6D",
		X"6F", X"72", X"79", X"20", X"45", X"72", X"61", X"73", X"65", X"64",
		X"0A", X"0D", X"0A", X"0D");
		
		--error - record/playback not enabled
	  type wordarrays_15 is array(0 to 38) of std_logic_vector(7 downto 0);
     constant err_rec_pl_not_enabled_words : wordarrays_15 := ( 
		X"45", X"72", X"72", X"6F", X"72", X"20", X"2D", X"20", X"52", X"65",
		X"63", X"6F", X"72", X"64", X"2F", X"70", X"6C", X"61", X"79", X"62",
		X"61", X"63", X"6B", X"20", X"4E", X"6F", X"74", X"20", X"45", X"6E",
		X"61", X"62", X"6C", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
		--error - erase not enabled
	  type wordarrays_16 is array(0 to 28) of std_logic_vector(7 downto 0);
     constant err_erase_not_enabled_words : wordarrays_16 := ( 
		X"45", X"72", X"72", X"6F", X"72", X"20", X"2D", X"20", X"45", X"72",
		X"61", X"73", X"65", X"20", X"4E", X"6F", X"74", X"20", X"45", X"6E",
		X"61", X"62", X"6C", X"65", X"64", X"0A", X"0D", X"0A", X"0D");
		
		--error - erase still enabled
	  type wordarrays_17 is array(0 to 30) of std_logic_vector(7 downto 0);
     constant err_erase_still_enabled_words : wordarrays_17 := ( 
		X"45", X"72", X"72", X"6F", X"72", X"20", X"2D", X"20", X"45", X"72",
		X"61", X"73", X"65", X"20", X"53", X"74", X"69", X"6C", X"6C", X"20",
		X"45", X"6E", X"61", X"62", X"6C", X"65", X"64", X"0A", X"0D", X"0A",
		X"0D");
		
		--error - record/playback still enabled
	  type wordarrays_18 is array(0 to 40) of std_logic_vector(7 downto 0);
     constant rec_pl_still_enabled_words : wordarrays_18 := ( 
		X"45", X"72", X"72", X"6F", X"72", X"20", X"2D", X"20", X"52", X"65",
		X"63", X"6F", X"72", X"64", X"2F", X"70", X"6C", X"61", X"79", X"62",
		X"61", X"63", X"6B", X"20", X"53", X"74", X"69", X"6C", X"6C", X"20",
		X"45", X"6E", X"61", X"62", X"6C", X"65", X"64", X"0A", X"0D", X"0A",
		X"0D");
		
		--error - can't play. turn mic off.
	  type wordarrays_19 is array(0 to 40) of std_logic_vector(7 downto 0);
     constant cant_play_mic_on_words : wordarrays_19 := ( 
		X"45", X"72", X"72", X"6F", X"72", X"20", X"2D", X"20", X"43", X"61",
		X"6E", X"27", X"74", X"20", X"70", X"6C", X"61", X"79", X"62", X"61",
		X"63", X"6B", X"2E", X"20", X"54", X"75", X"72", X"6E", X"20", X"6D",
		X"69", X"63", X"20", X"6F", X"66", X"66", X"2E", X"0A", X"0D", X"0A",
		X"0D");
		
		--error - already recording.
	  type wordarrays_20 is array(0 to 28) of std_logic_vector(7 downto 0);
     constant already_recording_words : wordarrays_20 := ( 
		X"45", X"72", X"72", X"6F", X"72", X"20", X"2D", X"20", X"41", X"6C",
		X"72", X"65", X"61", X"64", X"79", X"20", X"72", X"65", X"63", X"6F",
		X"72", X"64", X"69", X"6E", X"67", X"0A", X"0D", X"0A", X"0D");
		
		--can't record - in playback operation.
	  type wordarrays_21 is array(0 to 39) of std_logic_vector(7 downto 0);
     constant cant_rec_in_playback_words : wordarrays_21 := ( 
		X"43", X"61", X"6E", X"27", X"74", X"20", X"72", X"65", X"63", X"6F",
		X"72", X"64", X"20", X"2D", X"20", X"69", X"6E", X"20", X"70", X"6C",
		X"61", X"79", X"62", X"61", X"63", X"6B", X"20", X"6F", X"70", X"65",
		X"72", X"61", X"74", X"69", X"6F", X"6E", X"0A", X"0D", X"0A", X"0D");
		
		--can't change - in record process.
	  type wordarrays_22 is array(0 to 44) of std_logic_vector(7 downto 0);
     constant change_not_allowed_record_words : wordarrays_22 := ( 
		X"43", X"68", X"61", X"6E", X"67", X"65", X"20", X"6E", X"6F", X"74",
		X"20", X"61", X"6C", X"6C", X"6F", X"77", X"65", X"64", X"20", X"2D",
		X"20", X"69", X"6E", X"20", X"72", X"65", X"63", X"6F", X"72", X"64",
		X"69", X"6E", X"67", X"20", X"70", X"72", X"6F", X"63", X"65", X"73",
		X"73", X"0A", X"0D", X"0A", X"0D");
		
		--can't change - in playback process.
	  type wordarrays_23 is array(0 to 43) of std_logic_vector(7 downto 0);
     constant change_not_allowed_playback_words : wordarrays_23 := ( 
		X"43", X"68", X"61", X"6E", X"67", X"65", X"20", X"6E", X"6F", X"74", 
		X"20", X"61", X"6C", X"6C", X"6F", X"77", X"65", X"64", X"20", X"2D",
		X"20", X"69", X"6E", X"20", X"70", X"6C", X"61", X"79", X"62", X"61",
		X"63", X"6B", X"20", X"70", X"72", X"6F", X"63", X"65", X"73", X"73",
		X"0A", X"0D", X"0A", X"0D");
		
		--error - can't erase, nothing in memory
	  type wordarrays_24 is array(0 to 41) of std_logic_vector(7 downto 0);
     constant cant_erase_memory_empty_words : wordarrays_24 := ( 
		X"45", X"72", X"72", X"6F", X"72", X"20", X"2D", X"20", X"43", X"61",
		X"6E", X"27", X"74", X"20", X"65", X"72", X"61", X"73", X"65", X"2C",
		X"20", X"6E", X"6F", X"74", X"68", X"69", X"6E", X"67", X"20", X"69",
		X"6E", X"20", X"6D", X"65", X"6D", X"6F", X"72", X"79", X"0A", X"0D",
		X"0A", X"0D");
		
		--error - can't erase, currently in playback
	  type wordarrays_25 is array(0 to 38) of std_logic_vector(7 downto 0);
     constant cant_erase_in_playback_words : wordarrays_25 := ( 
		X"43", X"61", X"6E", X"27", X"74", X"20", X"65", X"72", X"61", X"73",
		X"65", X"20", X"2D", X"20", X"63", X"75", X"72", X"72", X"65", X"6E",
		X"74", X"6C", X"79", X"20", X"69", X"6E", X"20", X"70", X"6C", X"61",
		X"79", X"62", X"61", X"63", X"6B", X"0A", X"0D", X"0A", X"0D");
		
		--error - already in playback.
	  type wordarrays_26 is array(0 to 30) of std_logic_vector(7 downto 0);
     constant already_in_playback_words : wordarrays_26 := ( 
		X"45", X"72", X"72", X"6F", X"72", X"20", X"2D", X"20", X"61", X"6C", 
		X"72", X"65", X"61", X"64", X"79", X"20", X"69", X"6E", X"20", X"70",
		X"6C", X"61", X"79", X"62", X"61", X"63", X"6B", X"0A", X"0D", X"0A",
		X"0D");

	  signal outputlength, outputcounter, choiceout : integer := 0;
	  
	  --signals to represent the old inputs from the fpga switches and buttons
	  signal switch_in1_main, switch_in2_main, switch_in3_main,
	      switch_in4_main : std_logic := '0';
	  signal record_button_main, play_button_main, 
	     erase_button_main : std_logic := '1'; --board is pull down
	  
	  signal led1_state, led2_state, led3_state, led4_state : std_logic;
	  
	  signal rec_play_trigger : std_logic;
	  signal do_it_up : std_logic := '0';
	 
component xupv2pro_wrapper is	 
		port (
		clk_in : in std_logic;
		reset_in : in std_logic;
		Clk_Div : in std_logic;
		dcm_lock : in std_logic;
 
		-- CODEC signals
		AC97Reset_n : out std_logic;
		AC97Clk   : in  std_logic;   -- master clock for design
		Sync      : out std_logic;
		SData_Out : out std_logic;
		SData_In  : in  std_logic;
		switch_in1	  : in std_logic; --from switches
		switch_in2	  : in std_logic; 
		switch_in3	  : in std_logic; 
		switch_in4	  : in std_logic;
		record_button : in std_logic;  --from buttons
		play_button : in std_logic;
		erase_button : in std_logic;
		LED_OUTPUT1_main 	: out std_logic; --output to the led1
		LED_OUTPUT2_main 	: out std_logic; --output to the led2
		LED_OUTPUT3_main 	: out std_logic; --output to the led3
		LED_OUTPUT4_main 	: out std_logic; --output to the led4
		rec_play_trigger  : out std_logic;
		rec_play_ack      : in std_logic
		);
	end component;
	 
	  	  
begin

 wrapper_block : xupv2pro_wrapper
		port map (
		clk_in => ck,
		reset_in => rst,
		Clk_Div => div_clk,
		dcm_lock => dcm_lock,
 
		-- CODEC signals
		AC97Reset_n => AC97Reset_n,
		AC97Clk   => AC97Clk,
		Sync      => Sync,
		SData_Out => SData_Out,
		SData_In  => SData_In,
		switch_in1	  => switch_in1_main,
		switch_in2	  => switch_in2_main,
		switch_in3	  => switch_in3_main,
		switch_in4	  => switch_in4_main,
		record_button => record_button_main,
		play_button => play_button_main,
		erase_button => erase_button_main,
		LED_OUTPUT1_main 	=> led1_state,
		LED_OUTPUT2_main 	=> led2_state,
		LED_OUTPUT3_main 	=> led3_state,
		LED_OUTPUT4_main 	=> led4_state,
		rec_play_trigger  => rec_play_trigger,
		rec_play_ack      => do_it_up
		);


  process (ck,rst)
  
  begin 

    if (rst ='1') then
      -- Initialization of state
      state <= sa;
	   -- Initialization of UART
	   UART_addr_bus <= "00";
	   UART_data_bus <= "00000000";
	   UART_wr_rd <= '1';
	   UART_tr_cycle <='0';
		
	 	 
    elsif (ck='1' and ck'Event) then
	   
      case state is

	   when sa =>
			if UART_rx_int = '1' then
				state <= sb;
		   else			
				if rec_play_trigger = '1' and do_it_up = '0' then
				  rx_data <= X"99";  --fictional character
				  UART_addr_bus <= "00";
				  UART_wr_rd <= '0';
				  UART_tr_cycle <= '1';
				  state <= sd;
				elsif rec_play_trigger = '0' and do_it_up = '1' then
					do_it_up <= '0';
			   else
				  state <= sa;	-- Waiting until the UART is ready to RX
				end if;	
			end if;
			
	   when sb => 
		      state <= sc;   -- Delay to allow pin int up.

	   when sc =>
         
			if UART_rx_int = '1' then	-- Rx 
				state <= sd;    
				UART_addr_bus <= "00";
				UART_wr_rd <= '0';
				rx_data  <=  UART_data_bus_rx; 
				UART_tr_cycle <= '1';         
			else 
				state <= sc;
         end if;
	              
        when sd => 
		  
  --ok well the first thing we need to do is assign keys to each button
  --and switch that we used from the previous program to control the 
  --recording functions
  
  -- r = record button
  -- e = erase button
  -- p = playback button
  
  -- 1 = mic mute or unmute
  -- 2 = erase enable or disable
  -- 3 = record/playback enable or disable
		  
if UART_ack = '1' then  --Wait for ack
   if rx_data = X"99" then  --fictional letter
				if led4_state = '1' then
					--in playback
					outputlength <= wordarrays_9'length - 1;
					outputcounter <= 0;					
					choiceout <= 9;
					do_it_up <= '1';
					state <= s0;
				else
					--in record
					outputlength <= wordarrays_6'length - 1;
					outputcounter <= 0;
					choiceout <= 6;
					do_it_up <= '1';
					state <= s0;
				end if;  
	elsif rx_data = X"72" then -- Comparing for rx an "r"
	   if led3_state = '0' and led4_state = '0' then --already recording
			outputlength <= wordarrays_20'length - 1;
			outputcounter <= 0;
			choiceout <= 20;
			state <= s0;
		elsif led3_state = '0' and led4_state = '1' then --in playback operation
			outputlength <= wordarrays_21'length - 1;
			outputcounter <= 0;
			choiceout <= 21;
			state <= s0;
		elsif switch_in3_main = '0' then  --cant record, mic off
			outputlength <= wordarrays_12'length - 1;
			outputcounter <= 0;
			choiceout <= 12;
			state <= s0;
		elsif switch_in1_main = '0' then --record enable is off
			outputlength <= wordarrays_15'length - 1;
			outputcounter <= 0;
			choiceout <= 15;
			state <= s0;
		elsif switch_in2_main = '1' then --erase enable is still on
			outputlength <= wordarrays_17'length - 1;
			outputcounter <= 0;
			choiceout <= 17;
			state <= s0;
		elsif led1_state = '0' then --cant record, memory full
			outputlength <= wordarrays_13'length - 1;
			outputcounter <= 0;
			choiceout <= 13;
			state <= s0;
		else
			outputlength <= wordarrays_5'length - 1;
			outputcounter <= 0;
			choiceout <= 5;
			record_button_main <= '0';
			state <= s0;
	   end if;
		
		
   elsif rx_data = X"65" then -- Comparing for rx an "e"
	   if led3_state = '0' and led4_state = '0' then --already recording
			outputlength <= wordarrays_20'length - 1;
			outputcounter <= 0;
			choiceout <= 20;
			state <= s0;
		elsif led3_state = '0' then --cant erase, currently in playback
			outputlength <= wordarrays_25'length - 1;
			outputcounter <= 0;
			choiceout <= 25;
			state <= s0;		
		elsif switch_in2_main = '0' then  --erase not enabled
			outputlength <= wordarrays_16'length - 1;
			outputcounter <= 0;
			choiceout <= 16;
			state <= s0;
		elsif switch_in1_main = '1' then --record/playback still enabled
			outputlength <= wordarrays_18'length - 1;
			outputcounter <= 0;
			choiceout <= 18;
			state <= s0;
		elsif led1_state = '1' then --error - can't erase, nothing in memory
			outputlength <= wordarrays_24'length - 1;
			outputcounter <= 0;
			choiceout <= 24;
			state <= s0;		
		else
			outputlength <= wordarrays_7'length - 1;
			outputcounter <= 0;
			choiceout <= 7;
			erase_button_main <= '0';
			state <= s0;
      end if;		
		
   elsif rx_data = X"70" then -- Comparing for rx an "p"
	   if led3_state = '0' and led4_state = '1' then --already in playback
			outputlength <= wordarrays_26'length - 1;
			outputcounter <= 0;
			choiceout <= 26;
			state <= s0;
		elsif led3_state = '0' and led4_state = '0' then --already recording
			outputlength <= wordarrays_20'length - 1;
			outputcounter <= 0;
			choiceout <= 20;
			state <= s0;
		elsif switch_in1_main = '0' then  --record/playback not enabled
			outputlength <= wordarrays_15'length - 1;
			outputcounter <= 0;
			choiceout <= 15;
			state <= s0;
		elsif switch_in2_main = '1' then --erase still enabled
			outputlength <= wordarrays_17'length - 1;
			outputcounter <= 0;
			choiceout <= 17;
			state <= s0;
		elsif led4_state = '0' then --error - mic still on
			outputlength <= wordarrays_19'length - 1;
			outputcounter <= 0;
			choiceout <= 19;
			state <= s0;		
		elsif led2_state = '0' then --cant playback - memory empty
			outputlength <= wordarrays_14'length - 1;
			outputcounter <= 0;
			choiceout <= 14;
			state <= s0;
		else
			outputlength <= wordarrays_8'length - 1;
			outputcounter <= 0;
			choiceout <= 8;
			play_button_main <= '0';
			state <= s0;		
		end if;
	elsif rx_data = X"31" then -- Comparing for rx an "1"
	  if led3_state = '0' and led4_state = '0' then
				--currently recording, dont allow change
				outputlength <= wordarrays_22'length - 1;
				outputcounter <= 0;
				choiceout <= 22;
				state <= s0;
	  elsif led3_state = '0' and led4_state = '1' then
				--currently in playback, dont allow change
				outputlength <= wordarrays_23'length - 1;
				outputcounter <= 0;
				choiceout <= 23;
				state <= s0; 
	  					
	  else
			if switch_in3_main = '0' then
				switch_in3_main <= '1';
				outputlength <= wordarrays_10'length - 1;
				choiceout <= 10;
				outputcounter <= 0;
				state <= s0;
			
			else
				switch_in3_main <= '0';
				outputlength <= wordarrays_11'length - 1;
				choiceout <= 11;
				outputcounter <= 0;
				state <= s0;
			
			end if;
		
		end if;
	elsif rx_data = X"32" then -- Comparing for rx an "2"
	  if led3_state = '0' and led4_state = '0' then
				--currently recording, dont allow change
				outputlength <= wordarrays_22'length - 1;
				outputcounter <= 0;
				choiceout <= 22;
				state <= s0;
	  elsif led3_state = '0' and led4_state = '1' then
				--currently in playback, dont allow change
				outputlength <= wordarrays_23'length - 1;
				outputcounter <= 0;
				choiceout <= 23;
				state <= s0; 
					
		else
			if switch_in2_main = '0' then
				switch_in2_main <= '1';
				outputlength <= wordarrays_3'length - 1;
				choiceout <= 3;
				outputcounter <= 0;
				state <= s0;
			
			else
				switch_in2_main <= '0';
				outputlength <= wordarrays_4'length - 1;
				choiceout <= 4;
				outputcounter <= 0;
				state <= s0;
			
			end if;			
		end if;
	elsif rx_data = X"33" then -- Comparing for rx an "3"
	   if led3_state = '0' and led4_state = '0' then
				--currently recording, dont allow change
				outputlength <= wordarrays_22'length - 1;
				outputcounter <= 0;
				choiceout <= 22;
				state <= s0;
		elsif led3_state = '0' and led4_state = '1' then
				--currently in playback, dont allow change
				outputlength <= wordarrays_23'length - 1;
				outputcounter <= 0;
				choiceout <= 23;
				state <= s0; 					
		else		
			if switch_in1_main = '0' then
				switch_in1_main <= '1';
				outputlength <= wordarrays_1'length - 1;
				choiceout <= 1;
				outputcounter <= 0;
				state <= s0;
			
			else
				switch_in1_main <= '0';
				outputlength <= wordarrays_2'length - 1;
				choiceout <= 2;
				outputcounter <= 0;
				state <= s0;
				
			end if;				
	   end if;		
	else
	   state <= sa;
	end if;
	
 UART_tr_cycle <= '0';
  
else 
 state <= sd;
end if;	  
			   
	
		when s0 => state <= s1; 	-- Delay to allow pin int UART down
 
	   when s1 => -- Starting the machine. Checking for End of Conversion.

				 state <= st1_wait;
				 -- Keeping UART IDLE
			    UART_addr_bus <= "00";
	 		    UART_data_bus <= "00000000";
	 		    UART_wr_rd <= '1';
	 		    UART_tr_cycle <='0';
				 								    
	   when st1_wait =>
			if UART_tx_int = '1' then 
				state <= s2;
			else
				state <= st1_wait;
			end if;
		
		when s2 => state <= s3;  --Tx first nibble
	   		 UART_addr_bus <= "00"; 
			
			case choiceout is
			
			when 1 =>
				UART_data_bus <= record_en_words(outputcounter);
			when 2 =>
				UART_data_bus <= record_dis_words(outputcounter);
			when 3 =>
				UART_data_bus <= erase_en_words(outputcounter);
			when 4 =>
				UART_data_bus <= erase_dis_words(outputcounter);
			when 5 =>
				UART_data_bus <= record_start_words(outputcounter);
			when 6 =>
				UART_data_bus <= record_stop_words(outputcounter);
			when 7 =>
				UART_data_bus <= memory_erased_words(outputcounter);
			when 8 =>
				UART_data_bus <= play_started_words(outputcounter);
			when 9 =>
				UART_data_bus <= play_stopped_words(outputcounter);
			when 10 =>
				UART_data_bus <= mic_on_words(outputcounter);
			when 11 =>
				UART_data_bus <= mic_off_words(outputcounter);
			when 12 =>
				UART_data_bus <= cant_record_mic_words(outputcounter);
			when 13 =>
				UART_data_bus <= cant_record_memory_words(outputcounter);
			when 14 =>
				UART_data_bus <= cant_play_mem_erased_words(outputcounter);
			when 15 =>
				UART_data_bus <= err_rec_pl_not_enabled_words(outputcounter);
			when 16 =>
				UART_data_bus <= err_erase_not_enabled_words(outputcounter);
			when 17 =>
				UART_data_bus <= err_erase_still_enabled_words(outputcounter);
			when 18 =>
				UART_data_bus <= rec_pl_still_enabled_words(outputcounter);
			when 19 =>
				UART_data_bus <= cant_play_mic_on_words(outputcounter);
			when 20 =>
				UART_data_bus <= already_recording_words(outputcounter);
			when 21 =>
				UART_data_bus <= cant_rec_in_playback_words(outputcounter);
			when 22 =>
				UART_data_bus <= change_not_allowed_record_words(outputcounter);
			when 23 =>
				UART_data_bus <= change_not_allowed_playback_words(outputcounter);
			when 24 =>
				UART_data_bus <= cant_erase_memory_empty_words(outputcounter);
			when 25 =>
				UART_data_bus <= cant_erase_in_playback_words(outputcounter);
			when 26 =>
				UART_data_bus <= already_in_playback_words(outputcounter);
			when others =>
				--nothing here 
			end case;				
				
	 		    UART_wr_rd <= '1';
	 		    UART_tr_cycle <='1';  -- For WishBone protocol

	   when s3 => 
			if UART_ack = '1' then  -- Waiting for UART ack
				state <= s3delay;	 
			   UART_addr_bus <= "00";
	 		      
			case choiceout is
			
			when 1 =>
				UART_data_bus <= record_en_words(outputcounter);
			when 2 =>
				UART_data_bus <= record_dis_words(outputcounter);
			when 3 =>
				UART_data_bus <= erase_en_words(outputcounter);
			when 4 =>
				UART_data_bus <= erase_dis_words(outputcounter);
			when 5 =>
				UART_data_bus <= record_start_words(outputcounter);
			when 6 =>
				UART_data_bus <= record_stop_words(outputcounter);
			when 7 =>
				UART_data_bus <= memory_erased_words(outputcounter);
			when 8 =>
				UART_data_bus <= play_started_words(outputcounter);				
			when 9 =>
				UART_data_bus <= play_stopped_words(outputcounter);				
			when 10 =>
				UART_data_bus <= mic_on_words(outputcounter);
			when 11 =>
				UART_data_bus <= mic_off_words(outputcounter);
			when 12 =>
				UART_data_bus <= cant_record_mic_words(outputcounter);
			when 13 =>
				UART_data_bus <= cant_record_memory_words(outputcounter);
			when 14 =>
				UART_data_bus <= cant_play_mem_erased_words(outputcounter);
			when 15 =>
				UART_data_bus <= err_rec_pl_not_enabled_words(outputcounter);
			when 16 =>
				UART_data_bus <= err_erase_not_enabled_words(outputcounter);
			when 17 =>
				UART_data_bus <= err_erase_still_enabled_words(outputcounter);
			when 18 =>
				UART_data_bus <= rec_pl_still_enabled_words(outputcounter);
			when 19 =>
				UART_data_bus <= cant_play_mic_on_words(outputcounter);
			when 20 =>
				UART_data_bus <= already_recording_words(outputcounter);
			when 21 =>
				UART_data_bus <= cant_rec_in_playback_words(outputcounter);
			when 22 =>
				UART_data_bus <= change_not_allowed_record_words(outputcounter);
			when 23 =>
				UART_data_bus <= change_not_allowed_playback_words(outputcounter);
			when 24 =>
				UART_data_bus <= cant_erase_memory_empty_words(outputcounter);
			when 25 =>
				UART_data_bus <= cant_erase_in_playback_words(outputcounter);
			when 26 =>
				UART_data_bus <= already_in_playback_words(outputcounter);
			when others =>
				--nothing here
			end case;				
			    
	 		   UART_wr_rd <= '1';
	 		   UART_tr_cycle <='0'; -- For WishBone protocol
	   	else
			   state <= s3; -- Waiting until for UART confirmation of WishBone
         end if;

        when s3delay =>  --Delay
	              state <= s3NiTx; --Delay to allow pin int UART down.
					  outputcounter <= outputcounter + 1;

	   when s3NiTx => 
				if UART_tx_int = '1' and outputcounter <= outputlength then
				  state <= s0;
				elsif UART_tx_int = '1' and outputcounter > outputlength then
				  outputcounter <= 0;
				  state <= sa;
				  if record_button_main = '0' then
						record_button_main <= '1';
				  elsif play_button_main = '0' then
						play_button_main <= '1';
				  elsif erase_button_main = '0' then
						erase_button_main <= '1';
				  end if;
				else
			     state <= s3NiTx;	-- Waiting until the UART is ready to TX
            end if;

      end case;
    end if; 

  end process;
  
  
  --this is so that I can always look and see what state each led is in
  LED_OUTPUT1_main <= led1_state;
  LED_OUTPUT2_main <= led2_state;
  LED_OUTPUT3_main <= led3_state;
  LED_OUTPUT4_main <= led4_state;
    
 
end Behavioral;
