library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Interface is
    Port ( 
     CLK : in std_logic;
     RST : in std_logic;
	  div_clk : in std_logic;
	  dcm_lock : in std_logic;
	  rx_in :in std_logic;
	  tx_out : out std_logic;
			  
			  -- CODEC signals
    AC97Reset_n : out std_logic;
    AC97Clk   : in  std_logic;   -- master clock for design
    Sync      : out std_logic;
    SData_Out : out std_logic;
    SData_In  : in  std_logic;
	 
	 LED_OUTPUT1_main 	: out std_logic; --output to the led1
	 LED_OUTPUT2_main 	: out std_logic; --output to the led2
	 LED_OUTPUT3_main 	: out std_logic; --output to the led3
	 LED_OUTPUT4_main 	: out std_logic --output to the led4
		);
end Interface;

architecture Structural of Interface is


component Init_all is
    Port ( rst : in std_logic;
           ck : in std_logic;
			  div_clk : in std_logic;
			  dcm_lock : in std_logic;
		 		 --Signals for the UART
				UART_addr_bus: out std_logic_vector(1 downto 0);
				UART_data_bus: out std_logic_vector(7 downto 0);
				UART_data_bus_rx: in std_logic_vector(7 downto 0);
				UART_wr_rd: out std_logic;
				UART_tr_cycle: out std_logic;
				UART_tx_int: in std_logic;
				UART_rx_int: in std_logic;
				UART_ack: in std_logic;
		 
				-- CODEC signals
				AC97Reset_n : out std_logic;
				AC97Clk   : in  std_logic;   -- master clock for design
				Sync      : out std_logic;
				SData_Out : out std_logic;
				SData_In  : in  std_logic;
				
				LED_OUTPUT1_main 	: out std_logic; --output to the led1
				LED_OUTPUT2_main 	: out std_logic; --output to the led2
				LED_OUTPUT3_main 	: out std_logic; --output to the led3
				LED_OUTPUT4_main 	: out std_logic --output to the led4
	     );
end component;

component UART is
  port (
-- Wishbone signals
     WB_CLK_I : in  std_logic;  -- clock
     WB_RST_I : in  std_logic;  -- Reset input
     WB_ADR_I : in  std_logic_vector(1 downto 0); -- Adress bus          
     WB_DAT_I : in  std_logic_vector(7 downto 0); -- DataIn Bus
     WB_DAT_O : out std_logic_vector(7 downto 0); -- DataOut Bus
     WB_WE_I  : in  std_logic;  -- Write Enable
     WB_STB_I : in  std_logic;  -- Strobe
     WB_ACK_O : out std_logic;	-- Acknowledge
-- process signals     
     IntTx_O  : out std_logic;  -- Transmit interrupt: indicate waiting for Byte
     IntRx_O  : out std_logic;  -- Receive interrupt: indicate Byte received
     BR_Clk_I : in  std_logic;  -- Clock used for Transmit/Receive
     TxD_PAD_O: out std_logic;  -- Tx RS232 Line
     RxD_PAD_I: in  std_logic);  -- Rx RS232 Line     
end component;

Signal S_UART_addr_bus: std_logic_vector(1 downto 0);
Signal S_UART_data_bus: std_logic_vector(7 downto 0);
Signal S_UART_wr_rd: std_logic;
Signal S_UART_tr_cycle: std_logic;
Signal S_UART_tx_int: std_logic;
Signal S_UART_rx_int: std_logic;
Signal S_UART_ack: std_logic;
Signal Bus_data_out: std_logic_vector(7 downto 0);
Signal int_rx: std_logic;
Signal rx_out: std_logic;
Signal RST_H: std_logic;



begin

RST_H <= not (RST); -- FOR COMPATIBILITY WITH XUP BOARD!!!!!

Mach: Init_all
    port map( rst => RST_H,
              ck => CLK,
				  div_clk => div_clk,
				  dcm_lock => dcm_lock,
		    -- UART Signals
		    UART_addr_bus => S_UART_addr_bus,
		    UART_data_bus => S_UART_data_bus,
			 UART_data_bus_rx => Bus_data_out,
		    UART_wr_rd	=> S_UART_wr_rd,
		    UART_tr_cycle => S_UART_tr_cycle,
		    UART_tx_int => S_UART_tx_int,
			 UART_rx_int => int_rx,
		    UART_ack => S_UART_ack,
			 AC97Reset_n => AC97Reset_n,
			 AC97Clk   => AC97Clk,
			 Sync      => Sync,
			 SData_Out => SData_Out,
			 SData_In  => SData_In,
			 LED_OUTPUT1_main => LED_OUTPUT1_main,
			 LED_OUTPUT2_main => LED_OUTPUT2_main,
			 LED_OUTPUT3_main => LED_OUTPUT3_main,
			 LED_OUTPUT4_main => LED_OUTPUT4_main
		   );

mini_u: UART
  port map(
		  -- Wishbone signals
     	  WB_CLK_I => CLK,
     	  WB_RST_I => RST_H,
     	  WB_ADR_I => S_UART_addr_bus,
     	  WB_DAT_I => S_UART_data_bus,
     	  WB_DAT_O => Bus_data_out, --taked out
     	  WB_WE_I => S_UART_wr_rd,
     	  WB_STB_I => S_UART_tr_cycle,
     	  WB_ACK_O => S_UART_ack, --taked out
		  -- process signals     
     	  IntTx_O => S_UART_tx_int,
     	  IntRx_O => int_rx,  
     	  BR_Clk_I => CLK,
     	  TxD_PAD_O => tx_out,
     	  RxD_PAD_I =>	rx_in  
		); 

end Structural;
