library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity top_level_file is
    Port ( 
		CLK_100 : in std_logic;
		CLK_32 : in std_logic;
      RST : in std_logic;
		rx_in :in std_logic;
		tx_out : out std_logic;
		-- CODEC signals
		AC97Reset_n : out std_logic;
		AC97Clk   : in  std_logic;   -- master clock for design
		Sync      : out std_logic;
		SData_Out : out std_logic;
		SData_In  : in  std_logic;
	 
		LED_OUTPUT1_main 	: out std_logic; --output to the led1
		LED_OUTPUT2_main 	: out std_logic; --output to the led2
		LED_OUTPUT3_main 	: out std_logic; --output to the led3
		LED_OUTPUT4_main 	: out std_logic --output to the led4
		);
end top_level_file;

architecture top_level of top_level_file is

COMPONENT clock_dcm  --24 mhz signal for ac97 part
		PORT(
		CLKIN_IN : IN std_logic;
		RST_IN : IN std_logic;          
		CLKFX_OUT : OUT std_logic;
		CLKIN_IBUFG_OUT : OUT std_logic;
		CLK0_OUT : OUT std_logic;
		LOCKED_OUT : OUT std_logic
		);
		END COMPONENT;
		

COMPONENT external_clock  --100 mhz clock in 
	PORT(
		CLKIN_IN : IN std_logic;
		RST_IN : IN std_logic;          
		CLKFX_OUT : OUT std_logic;
		CLKIN_IBUFG_OUT : OUT std_logic;
		CLK0_OUT : OUT std_logic;
		LOCKED_OUT : OUT std_logic
		);
	END COMPONENT;

component Interface is
    Port ( 
           CLK : in std_logic;
           RST : in std_logic;
			  div_clk : in std_logic;
			  dcm_lock : in std_logic;
			  rx_in :in std_logic;
		     tx_out : out std_logic;
			  		  -- CODEC signals
				AC97Reset_n : out std_logic;
				AC97Clk   : in  std_logic;   -- master clock for design
				Sync      : out std_logic;
				SData_Out : out std_logic;
				SData_In  : in  std_logic;
	 
				LED_OUTPUT1_main 	: out std_logic; --output to the led1
				LED_OUTPUT2_main 	: out std_logic; --output to the led2
				LED_OUTPUT3_main 	: out std_logic; --output to the led3
				LED_OUTPUT4_main 	: out std_logic --output to the led4
		);
end component;

signal Clk_Div, dcm_lock : std_logic;
signal clk_in_100mhz : std_logic;

begin


Inst_clock_dcm: clock_dcm 
    PORT MAP(
		CLKIN_IN => CLK_32,
		RST_IN => not (RST),
		CLKFX_OUT => Clk_Div,
		CLKIN_IBUFG_OUT => open,
		CLK0_OUT => open,
		LOCKED_OUT => dcm_lock
	);
	
Inst_external_clock: external_clock 
	PORT MAP(
		CLKIN_IN => CLK_100,
		RST_IN => not (RST),
		CLKFX_OUT => clk_in_100mhz,
		CLKIN_IBUFG_OUT => open,
		CLK0_OUT => open,
		LOCKED_OUT => open
	);

Inst_interface: Interface 
    PORT MAP(
		CLK => clk_in_100mhz,
		RST => RST,
		div_clk => Clk_Div,
		dcm_lock => dcm_lock,
		rx_in => rx_in,
		tx_out => tx_out,
		AC97Reset_n => AC97Reset_n,
		AC97Clk   => AC97Clk,
		Sync      => Sync,
		SData_Out => SData_Out,
		SData_In  => SData_In,
	 	LED_OUTPUT1_main 	=> LED_OUTPUT1_main,
		LED_OUTPUT2_main 	=> LED_OUTPUT2_main,
		LED_OUTPUT3_main 	=> LED_OUTPUT3_main,
		LED_OUTPUT4_main 	=> LED_OUTPUT4_main
	);

end top_level;
