--modified by jason finn
--for ece 443
--professor suddarth
--november 28, 2007

-------------------------------------------------------------------------------
-- Filename:        top.vhd
--
-- Description:     Sample circuit for doing audio standalone
-- 
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--
-------------------------------------------------------------------------------
-- Author:          Mike Wirthlin
-- Revision:        $Revision: 1.2 $
-- Date:            $Date: 2005/02/14 23:50:33 $
--
-- History:
--
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

entity xupv2pro_wrapper is
  port (
    clk_in : in std_logic;
    reset_in : in std_logic;
	 Clk_Div : in std_logic;
	 dcm_lock : in std_logic;
 
    -- CODEC signals
    AC97Reset_n : out std_logic;
    AC97Clk   : in  std_logic;   -- master clock for design
    Sync      : out std_logic;
    SData_Out : out std_logic;
    SData_In  : in  std_logic;
	 switch_in1	  : in std_logic; --from switches
	 switch_in2	  : in std_logic; 
	 switch_in3	  : in std_logic; 
	 switch_in4	  : in std_logic;
	 record_button : in std_logic;  --from buttons
	 play_button : in std_logic;
	 erase_button : in std_logic;
	 LED_OUTPUT1_main 	: out std_logic; --output to the led1
	 LED_OUTPUT2_main 	: out std_logic; --output to the led2
	 LED_OUTPUT3_main 	: out std_logic; --output to the led3
	 LED_OUTPUT4_main 	: out std_logic; --output to the led4
	 rec_play_trigger    : out std_logic;
	 rec_play_ack        : in std_logic
    );

end xupv2pro_wrapper;



architecture imp of xupv2pro_wrapper is

   COMPONENT ac97_if is
   port (
            clkin : in std_logic;
            --Reset : in std_logic;
    
            -- All signals synchronous to clk_in
            PCM_Playback_Left: in std_logic_vector(15 downto 0);
            PCM_Playback_Right: in std_logic_vector(15 downto 0);
    
            PCM_Record_Left: out std_logic_vector(15 downto 0);
            PCM_Record_Right: out std_logic_vector(15 downto 0);
				PCM_Record_Valid : out std_logic;

    
            AC97Reset_n : out std_logic;        -- AC97Clk
    
            -- CODEC signals (synchronized to AC97Clk)
            AC97Clk   : in  std_logic;
            Sync      : out std_logic;
            SData_Out : out std_logic;
            SData_In  : in  std_logic
        );
   end component ac97_if;
		
	component mem_block IS
		port (
			addr: IN std_logic_VECTOR(16 downto 0);
			clk: IN std_logic;
			din: IN std_logic_VECTOR(15 downto 0);
			dout: OUT std_logic_VECTOR(15 downto 0);
			en: IN std_logic;
			we: IN std_logic);
	end component mem_block;


         signal new_sample : std_logic;
         signal audio_left_in, audio_left_temp : std_logic_Vector(15 downto 0) := "0000000000000000";
			signal audio_right_in, audio_right_temp : std_logic_Vector(15 downto 0) := "0000000000000000";
			signal audio_left_playback, audio_right_playback : std_logic_Vector(15 downto 0) := "0000000000000000";
         signal audio_left_out, audio_left_temp2, audio_left_temp3 : std_logic_Vector(15 downto 0) := "0000000000000000";
			signal audio_right_out, audio_right_temp2, audio_right_temp3 : std_logic_Vector(15 downto 0) := "0000000000000000";
			
			signal LED_OUTPUT1_temp, LED_OUTPUT3_temp, LED_OUTPUT4_temp : std_logic := '1';
			signal LED_OUTPUT2_temp : std_logic := '0';
 
         signal debug_i : std_logic_vector(3 downto 0);
         signal reset_i : std_logic;
			signal AC97Reset_n_i: std_logic;
			signal enable_i : std_logic;

         signal sync_i,sdata_out_i : std_logic;
			
			signal mem_addr : std_logic_vector(16 downto 0) := "00000000000000000";
			signal mem_clk, mem_write_enable : std_logic := '0';
			signal mem_datain, mem_dataout : std_logic_vector(15 downto 0) := "0000000000000000";
			signal mem_counter, playback_counter : integer := 0;
			signal mem_enable : std_logic := '1';
			
			signal clk_in_fake : std_logic;
			signal rec_trigger : std_logic := '0';


begin 

			
  AC97Reset_n <= AC97Reset_n_i;
  Sync <= sync_i;
  SData_Out <= SData_Out_i;
  enable_i <= dcm_lock;

  LED_OUTPUT1_main <= LED_OUTPUT1_temp;
  LED_OUTPUT2_main <= LED_OUTPUT2_temp;
  LED_OUTPUT3_main <= LED_OUTPUT3_temp;
  LED_OUTPUT4_main <= LED_OUTPUT4_temp;


  ac97_if_I : ac97_if
  port map (
      clkin => Clk_Div,
    
      PCM_Playback_Left   => audio_left_temp2, 
      PCM_Playback_Right  => audio_right_temp2, 

      PCM_Record_Left     => audio_left_temp,
      PCM_Record_Right    => audio_right_temp, 
		PCM_Record_Valid    => new_sample,
     
      AC97Reset_n         => AC97Reset_n_i,
      AC97Clk             => AC97Clk,
      Sync                => sync_i,
      SData_Out           => SData_Out_i,
      SData_In            => SData_In
    );
	 
	 Inst_mem: mem_block
	 PORT MAP(
		addr 	=> mem_addr,
		clk 	=> new_sample,
		din 	=> mem_datain,
		dout 	=> mem_dataout,
		en 	=> mem_enable,
		we 	=> mem_write_enable
	);

      
	with switch_in3 select
	  audio_left_temp2 <= audio_left_temp when '1',
								 audio_left_playback when '0',
								 "0000000000000000" when others;
	

	with switch_in3 select
	  audio_right_temp2 <= audio_right_temp when '1',
								  audio_right_playback when '0',
								 "0000000000000000" when others;
								 
	with switch_in3 select
	  LED_OUTPUT4_temp <= '0' when '1',
								 '1' when '0',
								 '0'when others;
						 
	
	process (new_sample)
		begin
		
		if new_sample'event and new_sample = '1' then
					
			--recording logic here
			if rec_play_ack = '1' then
				rec_trigger <= '0';
			elsif switch_in1 = '1' and record_button = '0' and mem_counter = 0 
				and switch_in3 = '1' then
				--start recording by pressing the "up" button
				
				mem_write_enable <= '1';  --enable memory write
				mem_datain <= audio_left_temp;  --put what to write in the data slot
				mem_addr <= conv_std_logic_vector(mem_counter,17);  --tell it what address
				
				LED_OUTPUT1_temp <= '1'; --led 1 is off
				LED_OUTPUT2_temp <= '1'; --led 2 is off
				LED_OUTPUT3_temp <= '0'; --led 3 is on	
					
				mem_counter <= mem_counter + 1;  --increment address counter for next time
			
			elsif switch_in1 = '1' and mem_counter > 0 and mem_counter < 125000
            and switch_in3 = '1'	then
				--this is the recording process
				
				mem_datain <= audio_left_temp;  --put what to write in the data slot
				mem_addr <= conv_std_logic_vector(mem_counter,17);  --tell it what address
			   
				if mem_counter = 124999 then
					rec_trigger <= '1';
				end if;
				
				mem_counter <= mem_counter + 1;
			
			elsif switch_in1 = '1' and mem_counter = 125000 then
				--memory is now full
				
				LED_OUTPUT1_temp <= '0';  --led 1 is on
				LED_OUTPUT2_temp <= '1';  --led 2 is off
				LED_OUTPUT3_temp <= '1';  --led 3 is off
				
				mem_write_enable <= '0';  --disable memory write, now read
				playback_counter <= 0;
								
				
			
			elsif switch_in2 = '1' and mem_counter = 125000 and erase_button = '0' then
				--erase the memory
				
				mem_counter <= 0;
				playback_counter <= 0;
										
				LED_OUTPUT1_temp <= '1';  --led 1 is off
				LED_OUTPUT2_temp <= '0';  --led 2 is on
				LED_OUTPUT3_temp <= '1';  --led 3 is off
			
			end if;
			
			--playback logic here
			
			if switch_in3 = '0' and playback_counter = 0 and 
				play_button = '0' and mem_counter = 125000 then
				--start playback of memory recorded when button pressed
				
				LED_OUTPUT1_temp <= '0';  --led 1 is on
				LED_OUTPUT2_temp <= '1';  --led 2 is off
				LED_OUTPUT3_temp <= '0';  --led 3 is on
				
				mem_addr <= conv_std_logic_vector(playback_counter,17);  --tell it what address
				audio_left_playback <= mem_dataout;
				audio_right_playback <= mem_dataout;
				
				playback_counter <= playback_counter + 1;
				
				
			elsif switch_in3 = '0' and playback_counter > 0 and 
				playback_counter < 125000 and mem_counter = 125000 then
				--playback the memory recorded 
				
				mem_addr <= conv_std_logic_vector(playback_counter,17);  --tell it what address
				audio_left_playback <= mem_dataout;
				audio_right_playback <= mem_dataout;
												
				LED_OUTPUT1_temp <= '0';  --led 1 is on
				LED_OUTPUT2_temp <= '1';  --led 2 is off
				LED_OUTPUT3_temp <= '0';  --led 3 is on
				
				if playback_counter = 124999 then
					rec_trigger <= '1';
				end if;
				
				playback_counter <= playback_counter + 1;
			
			elsif playback_counter = 125000 and switch_in3 = '0' 
				and mem_counter = 125000 then
				--reset the counter to playback again
				
				LED_OUTPUT1_temp <= '0';  --led 1 is on
				LED_OUTPUT2_temp <= '1';  --led 2 is off
				LED_OUTPUT3_temp <= '1';  --led 3 is off
				
				playback_counter <= 0;
			
			else
			
			end if;
		
				
		else
		
			--do nothing here
			
		end if;
		
   end process;
		
rec_play_trigger <= rec_trigger;
	
end architecture imp;
  
